@@ -1,4 +1,8 @@
+#ifdef __WIN__
+#include <io.h>
+#else
 #include <unistd.h>
+#endif
 #include <time.h>
 #include <string>
 #include <iostream>
@@ -21,18 +25,19 @@ using namespace std;
 /**
  * Opens the FAT resource
  */
-FatSystem::FatSystem(string filename_, unsigned long long globalOffset_)
+FatSystem::FatSystem(string filename_, unsigned long long globalOffset_, OutputFormatType outputFormat_)
     : strange(0),
-    filename(filename_),
-    globalOffset(globalOffset_),
-    totalSize(-1),
-    listDeleted(false),
-    statsComputed(false),
-    freeClusters(0),
-    cacheEnabled(false),
-    type(FAT32),
+      filename(filename_),
+      globalOffset(globalOffset_),
+      totalSize(-1),
+      listDeleted(false),
+      statsComputed(false),
+      freeClusters(0),
+      cacheEnabled(false),
+      type(FAT32),
     rootEntries(0)
 {
+    this->_outputFormat = outputFormat_;
     fd = open(filename.c_str(), O_RDONLY|O_LARGEFILE);
     writeMode = false;
 
@@ -51,7 +56,7 @@ void FatSystem::enableCache()
         for (int cluster=0; cluster<totalClusters; cluster++) {
             cache[cluster] = nextCluster(cluster);
         }
-        
+
         cacheEnabled = true;
     }
 }
@@ -67,7 +72,7 @@ void FatSystem::enableWrite()
 
         throw oss.str();
     }
-    
+
     writeMode = true;
 }
 
@@ -136,25 +141,27 @@ void FatSystem::parseHeader()
     reservedSectors = FAT_READ_SHORT(buffer, FAT_RESERVED_SECTORS)&0xffff;
     oemName = string(buffer+FAT_DISK_OEM, FAT_DISK_OEM_SIZE);
     fats = buffer[FAT_FATS];
-    
+
     sectorsPerFat = FAT_READ_SHORT(buffer, FAT16_SECTORS_PER_FAT)&0xffff;
 
     if (sectorsPerFat != 0) {
         type = FAT16;
-        bits = 16;
         diskLabel = string(buffer+FAT16_DISK_LABEL, FAT16_DISK_LABEL_SIZE);
         fsType = string(buffer+FAT16_DISK_FS, FAT16_DISK_FS_SIZE);
         rootEntries = FAT_READ_SHORT(buffer, FAT16_ROOT_ENTRIES)&0xffff;
         rootDirectory = 0;
 
-        if (trim(fsType) == "FAT12") {
-            bits = 12;
-        }
-
         totalSectors = FAT_READ_SHORT(buffer, FAT16_TOTAL_SECTORS)&0xffff;
         if (!totalSectors) {
             totalSectors = FAT_READ_LONG(buffer, FAT_TOTAL_SECTORS)&0xffffffff;
         }
+
+        unsigned int rootDirSectors =
+            (rootEntries*FAT_ENTRY_SIZE + bytesPerSector-1)/bytesPerSector;
+        unsigned long dataSectors = totalSectors -
+            (reservedSectors + fats*sectorsPerFat + rootDirSectors);
+        unsigned long totalClusters = dataSectors/sectorsPerCluster;
+        bits = (totalClusters > MAX_FAT12) ? 16 : 12;
     } else {
         type = FAT32;
         bits = 32;
@@ -175,9 +182,9 @@ void FatSystem::parseHeader()
         strange++;
     }
 
-    if (fats != 2) {
-        printf("WARNING: Fats number different of 2 (%llu)\n", fats);
-        strange++;
+    if (fats == 0) {
+        printf("WARNING: Fats numbner is 0\n");
+         strange++;
     }
 
     if (rootDirectory != 2 && type == FAT32) {
@@ -192,8 +199,14 @@ void FatSystem::parseHeader()
 unsigned int FatSystem::nextCluster(unsigned int cluster, int fat)
 {
     int bytes = (bits == 32 ? 4 : 2);
+#ifndef __WIN__
     char buffer[bytes];
-    
+#else
+    char *buffer = new char[bytes];
+    __try
+    {
+#endif
+
     if (!validCluster(cluster)) {
         return 0;
     }
@@ -205,7 +218,7 @@ unsigned int FatSystem::nextCluster(unsigned int cluster, int fat)
     readData(fatStart+fatSize*fat+(bits*cluster)/8, buffer, sizeof(buffer));
 
     unsigned int next;
-    
+
     if (type == FAT32) {
         next = FAT_READ_LONG(buffer, 0)&0x0fffffff;
 
@@ -236,22 +249,35 @@ unsigned int FatSystem::nextCluster(unsigned int cluster, int fat)
             }
         }
     }
+#ifdef __WIN__
 }
-    
+__finally
+{
+    delete[] buffer;
+}
+#endif
+}
+
 /**
  * Changes the next cluster in a file
  */
 bool FatSystem::writeNextCluster(unsigned int cluster, unsigned int next, int fat)
 {
     int bytes = (bits == 32 ? 4 : 2);
+#ifndef __WIN__
     char buffer[bytes];
+#else
+        char *buffer = new char[bytes];
+        __try
+        {
+#endif
 
     if (!validCluster(cluster)) {
         throw string("Trying to access a cluster outside bounds");
     }
 
     int offset = fatStart+fatSize*fat+(bits*cluster)/8;
-    
+
     if (bits == 12) {
         readData(offset, buffer, bytes);
         int bit = cluster*bits;
@@ -270,8 +296,15 @@ bool FatSystem::writeNextCluster(unsigned int cluster, unsigned int next, int fa
     }
 
     return writeData(offset, buffer, bytes) == bytes;
+#ifdef __WIN__
 }
-        
+__finally
+{
+    delete[] buffer;
+}
+#endif
+}
+
 bool FatSystem::validCluster(unsigned int cluster)
 {
     return cluster < totalClusters;
@@ -283,7 +316,7 @@ unsigned long long FatSystem::clusterAddress(unsigned int cluster, bool isRoot)
         cluster -= 2;
     }
 
-    int addr = (dataStart + bytesPerSector*sectorsPerCluster*cluster);
+    unsigned long long addr = (dataStart + bytesPerSector*sectorsPerCluster*cluster);
 
     if (type == FAT16 && !isRoot) {
         addr += rootEntries * FAT_ENTRY_SIZE;
@@ -394,7 +427,7 @@ vector<FatEntry> FatSystem::getEntries(unsigned int cluster, int *clusters, bool
         } else {
             cluster = nextCluster(cluster);
         }
-        
+
         if (clusters) {
             (*clusters)++;
         }
@@ -422,10 +455,10 @@ vector<FatEntry> FatSystem::getEntries(unsigned int cluster, int *clusters, bool
           }
         }
     } while (cluster != FAT_LAST);
-    
+
     return entries;
 }
-        
+
 void FatSystem::list(FatPath &path)
 {
     FatEntry entry;
@@ -439,54 +472,118 @@ void FatSystem::list(unsigned int cluster)
 {
     bool hasFree = false;
     vector<FatEntry> entries = getEntries(cluster, NULL, &hasFree);
-    printf("Directory cluster: %u\n", cluster);
-    if (hasFree) {
+    if(this->_outputFormat == Json)
+        cout << "{\"Cluster\":" << cluster << ",";
+    else
+        printf("Directory cluster: %u\n", cluster);
+    if (hasFree && this->_outputFormat != Json) {
         printf("Warning: this directory has free clusters that was read contiguously\n");
     }
+    if(this->_outputFormat == Json)
+        cout << "\"Entries\":";
     list(entries);
+    if(this->_outputFormat == Json)
+        cout << "}";
 }
-            
+
 void FatSystem::list(vector<FatEntry> &entries)
 {
     vector<FatEntry>::iterator it;
+    if(this->_outputFormat ==  Json){
+        cout << "[";
+        it = entries.begin();
+        FatEntry &first = *it;
+        for (; it!=entries.end(); it++) {
+            FatEntry &entry = *it;
 
-    for (it=entries.begin(); it!=entries.end(); it++) {
-        FatEntry &entry = *it;
+            if (entry.isErased() && !listDeleted) {
+                continue;
+            }
+            if(&entry != &first) {
+                cout << ",";
+            }
+            cout << "{\"EntryType\":";
+            if (entry.isDirectory()) {
+                cout << "\"Directory\",";
+            } else {
+                cout << "\"File\",";
+            }
 
-        if (entry.isErased() && !listDeleted) {
-            continue;
+            string name = entry.getFilename();
+            string shrtname = entry.getShortFilename();
+
+            cout << "\"EditDate\":\"" << entry.changeDate.isoFormat() << "\",";
+            cout << "\"Name\":\"" << name << "\",";
+            cout << "\"8DOT3Name\":\"" << shrtname << "\",";
+            cout << "\"Cluster\":" << entry.cluster << ",";
+
+            if (!entry.isDirectory()) {
+                string pretty = prettySize(entry.size);
+                cout << "\"Size\":" << entry.size << ",";
+                cout << "\"PrettySize\":\"" << pretty << "\",";
+            }
+
+            if (entry.isHidden()) {
+                cout << "\"IsHidden\":true,";
+            }
+            else {
+                cout << "\"IsHidden\":false,";
+            }
+            if (entry.isErased()) {
+                cout << "\"IsDeleted\":true";
+            }
+            else {
+                cout << "\"IsDeleted\":false";
+            }
+            cout << "}";            
+            fflush(stdout);
         }
+        cout << "]";
+    }
+    else {
+        for (it=entries.begin(); it!=entries.end(); it++) {
+            FatEntry &entry = *it;
 
-        if (entry.isDirectory()) {
-            printf("d");
-        } else {
-            printf("f");
+            if (entry.isErased() && !listDeleted) {
+                continue;
+            }
+
+            if (entry.isDirectory()) {
+                printf("d");
+            } else {
+                printf("f");
+            }
+
+            string name = entry.getFilename();
+            if (entry.isDirectory()) {
+                name += "/";
+            }
+
+            string shrtname = entry.getShortFilename();
+            if (name.compare(shrtname)) {
+                name += " (" + shrtname + ")";
+            }
+
+            printf(" %s ", entry.changeDate.pretty().c_str());
+            printf(" %-50s", name.c_str());
+
+            printf(" c=%u", entry.cluster);
+
+            if (!entry.isDirectory()) {
+                string pretty = prettySize(entry.size);
+                printf(" s=%llu (%s)", entry.size, pretty.c_str());
+            }
+
+            if (entry.isHidden()) {
+                printf(" h");
+            }
+            if (entry.isErased()) {
+                printf(" d");
+            }
+
+            printf("\n");
+            fflush(stdout);
         }
-
-        string name = entry.getFilename();
-        if (entry.isDirectory()) {
-            name += "/";
-        }
-
-        printf(" %s ", entry.changeDate.pretty().c_str());
-        printf(" %-30s", name.c_str());
-
-        printf(" c=%u", entry.cluster);
-        
-        if (!entry.isDirectory()) {
-            string pretty = prettySize(entry.size);
-            printf(" s=%llu (%s)", entry.size, pretty.c_str());
-        }
-
-        if (entry.isHidden()) {
-            printf(" h");
-        }
-        if (entry.isErased()) {
-            printf(" d");
-        }
-
-        printf("\n");
-        fflush(stdout);
     }
 }
 
@@ -504,7 +601,13 @@ void FatSystem::readFile(unsigned int cluster, unsigned int size, FILE *f, bool
         if (toRead > bytesPerCluster || size < 0) {
             toRead = bytesPerCluster;
         }
+#ifndef __WIN__
         char buffer[bytesPerCluster];
+#else
+                char *buffer = new char[bytesPerCluster];
+                __try
+                {
+#endif
         readData(clusterAddress(cluster), buffer, toRead);
 
         if (size != -1) {
@@ -540,9 +643,15 @@ void FatSystem::readFile(unsigned int cluster, unsigned int size, FILE *f, bool
                 cluster = currentCluster+1;
             }
         }
+#ifdef __WIN__
+    }
+    __finally
+    {
+        delete[] buffer;
+    }
+#endif
     }
 }
-
 bool FatSystem::init()
 {
     // Parsing header
@@ -564,44 +673,85 @@ bool FatSystem::init()
 
     return strange == 0;
 }
-        
+
 void FatSystem::infos()
 {
-    cout << "FAT Filesystem information" << endl << endl;
+    switch (this->_outputFormat) {
+        case Json: {
+            cout << "{\"FileSystemType\":\"" << trim(fsType) << "\",";
+            cout << "\"OemName\":\"" << oemName  << "\",";
+            cout << "\"TotalSectors\":" << totalSectors << ",";
+            cout << "\"TotalDataClusters\":" << totalClusters << ",";
+            cout << "\"DataSize\":" << dataSize << ",";
+            cout << "\"PrettyDataSize\":\"" << prettySize(dataSize) << "\",";
+            cout << "\"DiskSize\":" << totalSize << ",";
+            cout << "\"PrettyDiskSize\":\"" << prettySize(totalSize) << "\",";
+            cout << "\"BytesPerSector\":" << bytesPerSector << ",";
+            cout << "\"SectorsPerCluster\":" << sectorsPerCluster << ",";
+            cout << "\"BytesPerCluster\":" << bytesPerCluster << ",";
+            cout << "\"ReservedSectors\":" << reservedSectors << ",";
+            if (type == FAT16) {
+                cout << "\"RootEntries\":" << rootEntries << ",";
+                cout << "\"RootClusters\":" << rootClusters << ",";
+            }
+        cout << "\"SectorsPerFat\":" << sectorsPerFat << ",";
+            cout << "\"FatSize\":" << fatSize << ",";
+            cout << "\"PrettyFatSize\":\"" << prettySize(fatSize) << "\",";
+            cout << "\"Fat1StartAddress\":" << fatStart << ",";
+            cout << "\"Fat2StartAddress\":" << fatStart+fatSize << ",";
+            cout << "\"DataStartAddress\":" << dataStart << ",";
+            cout << "\"RootDirectoryCluster\":" << rootDirectory << ",";
+            cout << "\"DiskLabel\":\"" << trim(diskLabel) << "\",";
 
-    cout << "Filesystem type: " << fsType << endl;
-    cout << "OEM name: " << oemName << endl;
-    cout << "Total sectors: " << totalSectors << endl;
-    cout << "Total data clusters: " << totalClusters << endl;
-    cout << "Data size: " << dataSize << " (" << prettySize(dataSize) << ")" << endl;
-    cout << "Disk size: " << totalSize << " (" << prettySize(totalSize) << ")" << endl;
-    cout << "Bytes per sector: " << bytesPerSector << endl;
-    cout << "Sectors per cluster: " << sectorsPerCluster << endl;
-    cout << "Bytes per cluster: " << bytesPerCluster << endl;
-    cout << "Reserved sectors: " << reservedSectors << endl;
-    if (type == FAT16) {
-        cout << "Root entries: " << rootEntries << endl;
-        cout << "Root clusters: " << rootClusters << endl;
+            computeStats();
+            cout << "\"FreeClusters\":" << freeClusters << ",\"TotalClusters\":" << totalClusters << ",";
+            cout << "\"FreeClustersPercent\":" << (100*freeClusters/(double)totalClusters) << ",";
+            cout << "\"FreeSpace\":" << (freeClusters*bytesPerCluster) << ",";
+            cout << "\"PrettyFreeSpace\":\"" << prettySize(freeClusters*bytesPerCluster) << "\",";
+            cout << "\"UsedSpace\":" << ((totalClusters-freeClusters)*bytesPerCluster) << ",";
+            cout << "\"PrettyUsedSpace\":\"" << prettySize((totalClusters-freeClusters)*bytesPerCluster) << "\"}";
+            cout << endl;
+            break;
+        }
+        default: {
+            cout << "FAT Filesystem information" << endl << endl;
+
+            cout << "Filesystem type: " << fsType << endl;
+            cout << "OEM name: " << oemName << endl;
+            cout << "Total sectors: " << totalSectors << endl;
+            cout << "Total data clusters: " << totalClusters << endl;
+            cout << "Data size: " << dataSize << " (" << prettySize(dataSize) << ")" << endl;
+            cout << "Disk size: " << totalSize << " (" << prettySize(totalSize) << ")" << endl;
+            cout << "Bytes per sector: " << bytesPerSector << endl;
+            cout << "Sectors per cluster: " << sectorsPerCluster << endl;
+            cout << "Bytes per cluster: " << bytesPerCluster << endl;
+            cout << "Reserved sectors: " << reservedSectors << endl;
+            if (type == FAT16) {
+                cout << "Root entries: " << rootEntries << endl;
+                cout << "Root clusters: " << rootClusters << endl;
+            }
+            cout << "Sectors per FAT: " << sectorsPerFat << endl;
+            cout << "Fat size: " << fatSize << " (" << prettySize(fatSize) << ")" << endl;
+            printf("FAT1 start address: %016llx\n", fatStart);
+            printf("FAT2 start address: %016llx\n", fatStart+fatSize);
+            printf("Data start address: %016llx\n", dataStart);
+            cout << "Root directory cluster: " << rootDirectory << endl;
+            cout << "Disk label: " << diskLabel << endl;
+        cout << endl;
+
+            computeStats();
+            cout << "Free clusters: " << freeClusters << "/" << totalClusters;
+            cout << " (" << (100*freeClusters/(double)totalClusters) << "%)" << endl;
+            cout << "Free space: " << (freeClusters*bytesPerCluster) <<
+                " (" << prettySize(freeClusters*bytesPerCluster) << ")" << endl;
+            cout << "Used space: " << ((totalClusters-freeClusters)*bytesPerCluster) <<
+                " (" << prettySize((totalClusters-freeClusters)*bytesPerCluster) << ")" << endl;
+            cout << endl;
+            break;
+        }
     }
-    cout << "Sectors per FAT: " << sectorsPerFat << endl;
-    cout << "Fat size: " << fatSize << " (" << prettySize(fatSize) << ")" << endl;
-    printf("FAT1 start address: %016llx\n", fatStart);
-    printf("FAT2 start address: %016llx\n", fatStart+fatSize);
-    printf("Data start address: %016llx\n", dataStart);
-    cout << "Root directory cluster: " << rootDirectory << endl;
-    cout << "Disk label: " << diskLabel << endl;
-    cout << endl;
-
-    computeStats();
-    cout << "Free clusters: " << freeClusters << "/" << totalClusters;
-    cout << " (" << (100*freeClusters/(double)totalClusters) << "%)" << endl;
-    cout << "Free space: " << (freeClusters*bytesPerCluster) << 
-        " (" << prettySize(freeClusters*bytesPerCluster) << ")" << endl;
-    cout << "Used space: " << ((totalClusters-freeClusters)*bytesPerCluster) << 
-        " (" << prettySize((totalClusters-freeClusters)*bytesPerCluster) << ")" << endl;
-    cout << endl;
 }
-        
+
 bool FatSystem::findDirectory(FatPath &path, FatEntry &outputEntry)
 {
     int cluster;
@@ -665,11 +815,11 @@ bool FatSystem::findFile(FatPath &path, FatEntry &outputEntry)
 
     return found;
 }
-        
+
 void FatSystem::readFile(FatPath &path, FILE *f)
 {
     FatEntry entry;
-    
+
     if (findFile(path, entry)) {
         bool contiguous = false;
         if (entry.isErased() && freeCluster(entry.cluster)) {
@@ -679,12 +829,12 @@ void FatSystem::readFile(FatPath &path, FILE *f)
         readFile(entry.cluster, entry.size, f, contiguous);
     }
 }
-        
+
 void FatSystem::setListDeleted(bool listDeleted_)
 {
     listDeleted = listDeleted_;
 }
-        
+
 FatEntry FatSystem::rootEntry()
 {
     FatEntry entry;
@@ -699,7 +849,7 @@ bool FatSystem::freeCluster(unsigned int cluster)
 {
     return nextCluster(cluster) == 0;
 }
-        
+
 void FatSystem::computeStats()
 {
     if (statsComputed) {
@@ -722,7 +872,13 @@ void FatSystem::rewriteUnallocated(bool random)
     srand(time(NULL));
     for (int cluster=0; cluster<totalClusters; cluster++) {
         if (freeCluster(cluster)) {
+#ifndef __WIN__
             char buffer[bytesPerCluster];
+#else
+            char *buffer = new char[bytesPerCluster];
+            __try
+            {
+#endif
             for (int i=0; i<sizeof(buffer); i++) {
                 if (random) {
                     buffer[i] = rand()&0xff;
@@ -732,8 +888,15 @@ void FatSystem::rewriteUnallocated(bool random)
             }
             writeData(clusterAddress(cluster), buffer, sizeof(buffer));
             total++;
-        }
+#ifdef __WIN__
+            }
+            __finally
+            {
+                delete[] buffer;
+            }
+#endif
     }
+}
 
-    cout << "Scrambled " << total << " sectors" << endl;
+cout << "Scrambled " << total << " sectors" << endl;
 }
